import ipaddress
import markdown as md_package
import re
from flask import Blueprint

bp = Blueprint("filters", __name__)


def github_style_admonition(text):
    """Transform GitHub-style alerts (> [!NOTE], etc.) to custom HTML"""
    patterns = {
        r"> \[!NOTE\](.*?)(?:\n\n|\Z)": '<div class="markdown-alert markdown-alert-note"><p class="markdown-alert-title">Note</p>\\1</div>',
        r"> \[!TIP\](.*?)(?:\n\n|\Z)": '<div class="markdown-alert markdown-alert-tip"><p class="markdown-alert-title">Tip</p>\\1</div>',
        r"> \[!IMPORTANT\](.*?)(?:\n\n|\Z)": '<div class="markdown-alert markdown-alert-important"><p class="markdown-alert-title">Important</p>\\1</div>',
        r"> \[!WARNING\](.*?)(?:\n\n|\Z)": '<div class="markdown-alert markdown-alert-warning"><p class="markdown-alert-title">Warning</p>\\1</div>',
        r"> \[!CAUTION\](.*?)(?:\n\n|\Z)": '<div class="markdown-alert markdown-alert-caution"><p class="markdown-alert-title">Caution</p>\\1</div>',
    }

    for pattern, replacement in patterns.items():
        text = re.sub(pattern, replacement, text, flags=re.DOTALL)

    return text


@bp.app_template_filter("markdown")
def markdown_filter(text):
    """Convert markdown text to HTML with support for GitHub-style features"""
    if text:
        # Pre-process GitHub-style alerts
        text = github_style_admonition(text)

        # Convert to HTML with regular markdown
        html = md_package.markdown(
            text, extensions=["tables", "fenced_code", "codehilite", "nl2br"]
        )

        return html
    return ""


@bp.app_template_filter("ip_network")
def ip_network_filter(cidr):
    """Convert a CIDR string to an IP network object"""
    try:
        return ipaddress.ip_network(cidr, strict=False)
    except ValueError:
        return None


@bp.app_template_filter("ip_address")
def ip_address_filter(ip):
    """Convert an IP string to an IP address object"""
    try:
        return ipaddress.ip_address(ip)
    except ValueError:
        return None


@bp.app_template_global("get_ip_network")
def get_ip_network(cidr):
    """Global function to get an IP network object from CIDR"""
    try:
        return ipaddress.ip_network(cidr, strict=False)
    except ValueError:
        return None
